#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "huffman.h"

static void
dump_tree(const char *name, unsigned n, const struct huffman_node *tree)
{
  unsigned k;
  
  printf("const struct huffman_node %s[] = {", name);
  for (k = 0; k < n; k++)
    {
      if (!(k & 3))
	printf("\n  ");
      printf("{%d,%d,%d},", tree[k].zeros, tree[k].leaf, tree[k].value);
    }
  printf("\n};\n");
}

/* Includes the two unused codes 286 and 287. */
#define CODE_SIZE 288

int
main (int argc, char **argv)
{
  struct huffman_node tree[HUFFMAN_SIZE(CODE_SIZE)];
  unsigned char clen[CODE_SIZE];

  /* Literal and length code. */
  memset(clen, 8, 144);
  memset(clen + 144, 9, 256 - 144);
  memset(clen + 256, 7, 280 - 256);
  memset(clen + 280, 8, CODE_SIZE - 280);

  if (!huffman_tree (tree, CODE_SIZE, clen))
    {
      fprintf(stderr, "Failed to generate length/literal code.\n");
      return EXIT_FAILURE;
    }
     
  dump_tree ("fixed_length_tree", HUFFMAN_SIZE(CODE_SIZE), tree);
  
  memset(clen, 5, 32);

  if (!huffman_tree (tree, 32, clen))
    {
      fprintf(stderr, "Failed to generate distance code.\n");
      return EXIT_FAILURE;
    }

  dump_tree ("fixed_distance_tree", HUFFMAN_SIZE(32), tree);
  
  return 0;
};
